/**
   NOTE: This parser does not "do" anything. It does not create a
   a parse tree nor an abstract syntax tree. This parser is just a
   "recognizer" for the language. It will throw an exception if
   a string cannot be parsed.


   The static method parse() is a recursive descent parser for
   strings that hold the prefix notation of an arithmetic expression.

   Here is the BNF grammar for prefix notation expressions.

      Prefix -> BinOp Prefix Prefix
              | UnOp Prefix
              | Number

       BinOp -> '+' | '-' | '*' | '/' | '^'

       UnOp  -> 'neg' | 'sqrt'

      Number -> any valid Java double

   In a recursive descent parser, every production is handled by a method.
   If a production is recursive, then the method will be recursive.
   If two productions are mutually recursive, then the two methods will
   be mutually recursive. Since this grammar has four productions, the
   parser consists of four methods, getPrefix(), getBinOP(), getUnOp(),
   and getNumber().

   Input strings are "tokenized" by the very bare bones tokenizer
   contained in the file Tokenizer.java. This tokenizer requires
   that all tokens in the input string be surrounded by white space
   (which makes the tokenizer almost trivial to write).
*/

public class Parse
{
   public static void parse(String prefix) throws ParseException
   {
      Tokenizer tokens = new Tokenizer(prefix); // defined in Tokenizer.java

      getPrefix(tokens);        // parse the token stream

      if ( tokens.hasToken() )  // there shouldn't be any more tokens
         throw new ParseException("unexpected input: " + tokens.peekToken()
                                                       + "\n" + tokens);
   }//parse()



   private static void getPrefix(Tokenizer tokens) throws ParseException
   {
      String token = null;

      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // one token look ahead
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( "+-*/^".indexOf(token) != -1 ) // binary operator case
      {
         getBinOp(tokens);  // parse the operator
         getPrefix(tokens); // parse the first operand
         getPrefix(tokens); // parse the second operand
      }
      else if ( "neg sqrt".indexOf(token) != -1 ) // unary operator case
      {
         getUnOp(tokens);   // parse the operator
         getPrefix(tokens); // parse the (only) operand
      }
      else // operand case (this case terminates the recursion)
      {
         getNumber(tokens); // parse the number
      }
    }//getPrefix()


   private static void getBinOp(Tokenizer tokens)
   {
      // consume one token
      tokens.nextToken();
   }//getBinOp()


   private static void getUnOp(Tokenizer tokens)
   {
      // consume one token
      tokens.nextToken();
   }//getUnOp()


   private static void getNumber(Tokenizer tokens) throws NumberFormatException
   {
      // consume one token
      String token = tokens.nextToken();
      Double.parseDouble(token); // throws NumberFormatException
   }//getNumber()

}//Parse
