/**
   NOTE: This parser creates an abstract syntax tree (or, an "expression
   tree") for each prefix expression. An abstract syntax tree is much
   less cluttered than a full "parse tree". Whereas a parse tree gives
   you the "syntactic" structure of an expression, an abstract syntax
   tree gives you the "semantic" structure of the expression.

   The static method parse() is a recursive descent parser for
   strings that hold the prefix notation of an arithmetic expression.

   Here is the BNF grammar for prefix notation expressions.

      Prefix -> BinOp Prefix Prefix
              | UnOp Prefix
              | Number

       BinOp -> '+' | '-' | '*' | '/' | '^'

       UnOp  -> 'neg' | 'sqrt'

      Number -> any valid Java double

   In a recursive descent parser, every production is handled by a method.
   If a production is recursive, then the method will be recursive.
   If two productions are mutually recursive, then the two methods will
   be mutually recursive. Since this grammar has four productions, the
   parser consists of four methods, getPrefix(), getBinOP(), getUnOp(),
   and getNumber().

   Input strings are "tokenized" by the very bare bones tokenizer
   contained in the file Tokenizer.java. This tokenizer requires
   that all tokens in the input string be surrounded by white space
   (which makes the tokenizer almost trivial to write).
*/

public class Parse2AST
{
   public static Tree parse(String prefix) throws ParseException
   {
      Tokenizer tokens = new Tokenizer(prefix); // defined in Tokenizer.java

      Tree result = getPrefix(tokens);         // parse the token stream

      if ( tokens.hasToken() )  // there shouldn't be any more tokens
         throw new ParseException("unexpected input: " + tokens.peekToken()
                                                       + "\n" + tokens);
      return result;
   }//parse()



   private static Tree getPrefix(Tokenizer tokens) throws ParseException
   {
      Tree result = null;

      String token = null;

      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // one token look ahead
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( "+-*/^".indexOf(token) != -1 ) // binary operator case
      {
         result = new Tree( getBinOp(tokens),
                            getPrefix(tokens),
                            getPrefix(tokens) );
      }
      else if ( "neg sqrt".indexOf(token) != -1 ) // unary operator case
      {
         result = new Tree( getUnOp(tokens),
                            getPrefix(tokens) );
      }
      else // operand case (this case terminates the recursion)
      {
         result = new Tree( getNumber(tokens) );
      }
      return result;
   }//getPrefix()


   private static String getBinOp(Tokenizer tokens)
   {
      return tokens.nextToken(); // consume one token

   }//getBinOp()


   private static String getUnOp(Tokenizer tokens)
   {
      return tokens.nextToken(); // consume one token

   }//getUnOp()


   private static String getNumber(Tokenizer tokens)
   {
      return tokens.nextToken(); // consume one token

   }//getNumber()

}//Parse2AST
