/**
   NOTE: This parser creates the full parse tree (or
   "syntax tree") for each prefix expression, instead
   of the more compact abstract syntax tree (or
   "expression tree"),


   The static method parse() is a recursive descent parser for
   strings that hold the prefix notation of an arithmetic expression.

   Here is the BNF grammar for prefix notation expressions.

      Prefix -> BinOp Prefix Prefix
              | UnOp Prefix
              | Number

       BinOp -> '+' | '-' | '*' | '/' | '^'

       UnOp  -> 'neg' | 'sqrt'

      Number -> any valid Java double

   In a recursive descent parser, every production is handled by a method.
   If a production is recursive, then the method will be recursive.
   If two productions are mutually recursive, then the two methods will
   be mutually recursive. Since this grammar has four productions, the
   parser consists of four methods, getPrefix(), getBinOP(), getUnOp(),
   and getNumber().

   Input strings are "tokenized" by the very bare bones tokenizer
   contained in the file Tokenizer.java. This tokenizer requires
   that all tokens in the input string be surrounded by white space
   (which makes the tokenizer almost trivial to write).
*/

public class Parse2ParseTree
{
   public static Tree parse(String prefix) throws ParseException
   {
      Tokenizer tokens = new Tokenizer(prefix); // defined in Tokenizer.java

      Tree result = getPrefix(tokens);         // parse the token stream

      if ( tokens.hasToken() )  // there shouldn't be any more tokens
         throw new ParseException("unexpected input: " + tokens.peekToken()
                                                       + "\n" + tokens);
      return result;
   }//parse()



   private static Tree getPrefix(Tokenizer tokens) throws ParseException
   {
      Tree result = new Tree( "Prefix" ); // node that represents the grammar nonterminal

      String token = null;

      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // one token look ahead
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( -1 != "+-*/^".indexOf(token) ) // binary operator case
      {
         result.addSubTree( getBinOp(tokens) );  // subtree for the operator token
         result.addSubTree( getPrefix(tokens) ); // subtree for the first operand
         result.addSubTree( getPrefix(tokens) ); // subtree for the second operand
      }
      else if ( -1 != "neg sqrt".indexOf(token) ) // unary operator case
      {
         result.addSubTree( getUnOp(tokens) );   // subtree for the operator token
         result.addSubTree( getPrefix(tokens) ); // subtree for the (only) operand
      }
      else // operand case (this case terminates the recursion)
      {
         result.addSubTree( getNumber(tokens) );
      }
      return result;
   }//getPrefix()


   private static Tree getBinOp(Tokenizer tokens)
   {
      Tree result = new Tree( "BinOp" );

      String token = tokens.nextToken(); // consume one token

      result.addSubTree( new Tree(token) ); // subtree for the operator token

      return result;
   }//getBinOp()


   private static Tree getUnOp(Tokenizer tokens)
   {
      Tree result = new Tree( "UnOp" );

      String token = tokens.nextToken(); // consume one token

      result.addSubTree( new Tree(token) ); // subtree for the operator token

      return result;
   }//getUnOp()


   private static Tree getNumber(Tokenizer tokens)
   {
      Tree result = new Tree( "Number" );

      String token = tokens.nextToken(); // consume one token

      Double.parseDouble(token); // throws NumberFormatException

      result.addSubTree( new Tree(token) ); // subtree for the number token

      return result;
   }//getNumber()

}//Parse2ParseTree
