/**
   This parser does not "do" anything. It does not create a parse tree
   nor an abstract syntax tree. This parser is just a "recognizer" for
   the language. It will throw an exception if a string cannot be parsed.

   The static method parse() is a recursive descent parser for JSON strings.

   Here is a EBNF grammar for JSON strings.

 object -> '{' [ pair ( ',' pair)* ] '}'

   pair -> string ':' value

  array -> '[' [ value ( ',' value)* ] ']'

  value -> object   // begins with '{'
         | array    // begins with '['
         | true
         | false
         | null
         | string
         | number
*/

public class ParseJSON
{
   public static void parse(String prefix) throws ParseException
   {
      Tokenizer tokens = new Tokenizer(prefix);

      getObject(tokens);        // parse the token stream

      if ( tokens.hasToken() )  // there shouldn't be any more tokens
         throw new ParseException("unexpected input: "+ tokens.peekToken() + "\n" + tokens);
   }//parse()



   private static void getObject(Tokenizer tokens) throws ParseException
   {
      String token = null;

      if ( tokens.hasToken() )
      {
         token = tokens.nextToken();
         if ( ! token.equals("{") )
            throw new ParseException("getObject() expected '{' found '" + token + "'");
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // one token look ahead
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( ! token.equals("}") )
      {
         getPair(tokens);

         if ( tokens.hasToken() )
         {
            token = tokens.peekToken();  // one token look ahead
         }
         else
            throw new ParseException("unexpected end of input: " + "\n" + tokens);

         while ( token.equals(",") )
         {
            tokens.nextToken();  // consume the comma
            getPair(tokens);
            if ( tokens.hasToken() )
            {
               token = tokens.peekToken();  // one token look ahead
            }
            else
               throw new ParseException("unexpected end of input: " + "\n" + tokens);
         }
      }

      token = tokens.nextToken();
      if ( ! token.equals("}") )
         throw new ParseException("getObject() expected ',' or '}' found '" + token + "'");

    }//getObject()


   private static void getPair(Tokenizer tokens) throws ParseException
   {
      if ( tokens.hasToken() )
      {
         tokens.nextToken();  // consume the string
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( tokens.hasToken() )
      {
         String token = tokens.nextToken();
         if ( ! token.equals(":") )
            throw new ParseException("getPair() expected ':' found '" + token + "'");
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( tokens.hasToken() )
      {
         getValue(tokens);
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

   }//getPair()


   private static void getArray(Tokenizer tokens) throws ParseException
   {
      String token = null;

      if ( tokens.hasToken() )
      {
         token = tokens.nextToken();
         if ( ! token.equals("[") )
            throw new ParseException("getArray() expected '[' found '" + token + "'");
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // one token look ahead
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( ! token.equals("]") )
      {
         getValue(tokens);

         if ( tokens.hasToken() )
         {
            token = tokens.peekToken();  // one token look ahead
         }
         else
            throw new ParseException("unexpected end of input: " + "\n" + tokens);

         while ( token.equals(",") )
         {
            tokens.nextToken();  // consume the comma
            getValue(tokens);
            if ( tokens.hasToken() )
            {
               token = tokens.peekToken();  // one token look ahead
            }
            else
               throw new ParseException("unexpected end of input: " + "\n" + tokens);
         }
      }

      token = tokens.nextToken();
      if ( ! token.equals("]") )
         throw new ParseException("getArray() expected ',' or ']' found '" + token + "'");
   }//getArray()


   private static void getValue(Tokenizer tokens) throws ParseException
   {
      String token = null;

      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // one token look ahead
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( token.equals("{") )
      {
         getObject(tokens);
      }
      else if ( token.equals("[") )
      {
         getArray(tokens);
      }
      else if ( token.equals("true") )
      {
         tokens.nextToken();    // consume the literal
      }
      else if ( token.equals("false") )
      {
         tokens.nextToken();     // consume the literal
      }
      else if ( token.equals("null") )
      {
         tokens.nextToken();     // consume the literal
      }
      else  // number or string
      {
         tokens.nextToken();
      }

   }//getValue()

}//ParseJSON
