/**
   This program produces a full parse tree instead of
   an abstract syntax tree as in example Parse_16.java.

   L = { a, a+a, a*a, a+a+a, a+a*a, a*a+a, a*a*a, ... }

   '*' has higher precedence than '+',
   '+' is left associative,
   '*' is right associative.

    BNF:   expr -> expr '+' term | term   // expr is left recursive
           term -> 'a' '*' term | 'a'     // term is right recursive

    EBNF:  expr -> term ( '+' term )*     // should parse as left associative
           term -> 'a' [ '*' term ]       // term is right recursive

   NOTE: We use the EBNF grammar to write the parser that builds
         the parse tree, but the parser should build the parse tree
         for the BNF grammar, NOT the EBNF grammar. The EBNF grammar
         does have its own parse tree, but its parse tree, by itself,
         does not specify left or right associativity. This is because
         the EBNF grammar can be derived from either a left or right
         recursive BNF grammar, so the EBNF grammar, by itself, does
         not specify associativity.
*/

public class Parser_16_ParseTree
{
   public static Tree parse(String stringInL4)
   {
      Tokenizer tokens = new Tokenizer(stringInL4);

      return getExpr(tokens);
   }//parse()


   public static Tree getExpr(Tokenizer tokens)
   {
      Tree result = new Tree( "Expr" );
      result.addSubTree(getTerm(tokens));         // consume a sub expression
      while( tokens.hasToken() && tokens.peekToken().equals("+") )
      {
         result = new Tree("Expr",
                           result,                    // old tree on the left
                           new Tree(tokens.nextToken()),     // consume a '+'
                           getTerm(tokens));      // consume a sub expression
      }
      return result;
   }//getExpr()


   public static Tree getTerm(Tokenizer tokens)
   {
      Tree result = new Tree( "Term" );;
      result.addSubTree( new Tree(tokens.nextToken()) ); // consume a 'a'
      if( tokens.hasToken() && tokens.peekToken().equals("*") )
      {
         result.addSubTree( new Tree(tokens.nextToken()) ); // consume the '*'
         result.addSubTree( getTerm(tokens) );              // consume a sub expression
      }
      return result;
   }//getTerm()



   public static void main(String[] args)
   {
      String[] stringsInL = { "a",
                              "a + a",
                              "a * a",
                              "a + b + c",
                              "a + b * c",
                              "a * b + c",
                              "a * a * a",
                              "a + b + c + e + f + g + h",  // sum of 7 terms
                              "a + b + c * e * f + g + h",  // sum of 5 terms
                              "a + b + c + e * f * g * h",  // sum of 4 terms
                              "a * b * c * e + f + g + h",  // sum of 4 terms
                              "a * b * c + e + f * g * h",  // sum of 3 terms
                              "a * b * c * e * f * g * h" };

      for (int i = 0; i < stringsInL.length; i++)
      {
         Tree parseTree = parse(stringsInL[i]);

         try  // Create dot and png files from the parse tree.
         {
            // Create the (empty) dot file.
            String baseName = String.format("Parser_16_%02d_ParseTree", i);
            java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot") );
            // Put dot commands into the dot file
            out.println("//  " + parseTree + "\n");
            out.println( Tree2dot.tree2dot(parseTree) + "\n" );
            out.close();
            // Create a command line for running dot.exe.
            String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }//main()

}//Parser_16_ParseTree
