/**
   Here is an unambiguous, "right associative", BNF grammar
   (and also an EBNF grammar) for the language

      L = { a, a+a, a+a+a, a+a+a+a, ... }.


   BNF:    expr -> 'a' '+' expr | 'a'  // this is right recursive


   EBNF:   expr -> 'a' [ '+' expr ]    // this is still right recursive


   Using the BNF grammar we parse the string "a+a+a+a" as if it were
   "grouped" as a+(a+(a+a)).

   Below is a recursive descent parser based on the EBNF grammar. Since
   there is only one production in the grammar, the parser is a single
   method, getExpr().

   Notice how the "or" symbol in the BNF (and the "option" symbol in the
   EBNF) becomes an if-statement in the parser. If the token stream has
   another token, it is checked to be a '+' operator, then it is read
   from the stream and another expression is consumed recursively.

   This parser builds the expression tree from the bottom up. Notice how
   the constructor call in line 52 builds the final expression tree by
   making a recursive call to getExpr() and using the returned tree as
   the right sub-tree of the final expression tree. The constructor call
   in line 52 cannot complete the building of the final tree until the
   recusive call returns. So the "bottom" of the expression tree must be
   completely build before the "top" of the tree can be completed.
*/

public class Parser_2
{
   public static Tree parse(String stringInL)
   {
      Tokenizer tokens = new Tokenizer(stringInL);

      return getExpr(tokens);  // start the parser
   }//parse()


   public static Tree getExpr(Tokenizer tokens)
   {
      String token = tokens.nextToken();  // consume a 'a'
      Tree result = new Tree( token );
      if ( tokens.hasToken() && tokens.peekToken().equals("+") )
      {
         result = new Tree(tokens.nextToken(), // consume the '+'
                           result,
                           getExpr(tokens)); //recursively consume another expr
      }
      return result;
   }//getExpr()



   public static void main(String[] args)
   {
      String[] stringsInL = { "a",
                              "a + a",
                              "a + a + a",
                              "a + a + a + a",
                              "a + b + c + d" };

      for (int i = 0; i < stringsInL.length; i++)
      {
         Tree parseTree = parse(stringsInL[i]);

         try  // Create dot and png files from the AST.
         {
            // Create the (empty) dot file.
            String baseName = String.format("Parser_2_%02d_AST", i);
            java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot") );
            // Put dot commands into the dot file
            out.println("//  " + parseTree + "\n");
            out.println( Tree2dot.tree2dot(parseTree) + "\n" );
            out.close();
            // Create a command line for running dot.exe.
            String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }//main()

}//Parser_2


/*


*/
