/**
   Here is a grammar for a language with two operators
   and parentheses

   L = { a, a+a, a*a, a+(a+a), (a+a)*a, a*a+a, (a*a)*a, ... }

   which gives '*' higher precedence than '+',
   makes '+' left associative,
   and makes '*' right associative.

    BNF:   expr -> expr '+' term | term       // expr is left recursive
           term -> factor '*' term | factor   // term is right recursive
         factor -> 'a' | '(' expr ')'


    EBNF:  expr -> term ( '+' term )*        // expr is now iterative
           term -> factor [ '*' term ]       // term is still right recursive
         factor -> 'a' | '(' expr ')'

   Notice that getExpr() is the same in this parser as in Parser_16,
               getTerm() is almost the same as getTerm() from Parser_16,
               getFactor() is the same as getTerm() from Parser_6.
*/

public class Parser_25
{
   public static Tree parse(String stringInL)
   {
      Tokenizer tokens = new Tokenizer(stringInL);

      return getExpr(tokens);
   }//parse()


   public static Tree getExpr(Tokenizer tokens)
   {
      Tree result = getTerm(tokens);           // consume a sub expression
      while( tokens.hasToken() && tokens.peekToken().equals("+") )
      {
         result = new Tree(tokens.nextToken(), // consume the '+'
                           result,             // old tree on the left
                           getTerm(tokens));   // consume a sub expression
      }
      return result;
   }//getExpr()


   public static Tree getTerm(Tokenizer tokens)
   {
      Tree result = getFactor(tokens);
      if( tokens.hasToken() && tokens.peekToken().equals("*") )
      {
         String tk = tokens.nextToken();                 // consume the '*'
         result = new Tree(tk, result, getTerm(tokens)); // consume a sub expression
      }
      return result;
   }//getTerm()


   public static Tree getFactor(Tokenizer tokens)
   {
      Tree result = null;
      String tk = tokens.peekToken();
      if( tk.equals("(") )             // conditionally consume more tokens
      {
         tokens.match("(");        // consume a '('
         result = getExpr(tokens); // consume a sub expression
         tokens.match(")");        // consume a ')'
      }
      else
      {
         result = new Tree(tokens.nextToken()); // consume the 'a'
      }
      return result;
   }//getFactor()


   public static void main(String[] args)
   {
      String[] stringsInL = { "a + b + c",
                              "a + ( b + c )",
                              "a * b + c",
                              "a * ( b + c )",
                              "a * b * c",
                              "( a * b ) * c",
                              "a + b + ( c + e + f ) + g + h",     // sum of 5 terms
                              "a + ( b + c ) * e * ( f + g + h )", // sum of 2 terms
                              "( a + b + c + e ) * f * g * h",     // product of 4 factors
                              "a * b * ( c * e + f + g + h ) ",    // product of 3 factors
                              "a * b * c + e + f * g * h",         // sum of 3 terms
                              "a * b * c * e * f * g * h",         // product of 7 factors
                              "a + b + c * d",     // put in lots of redundant parentheses
                              "( a + b ) + c * d",
                              "( a + b ) + ( c * d )",
                              "( a + b ) + ( ( c * d ) )",
                              "( ( a + b ) + ( ( c * d ) ) )",
                              "( ( ( a ) + ( b ) ) + ( ( ( c ) * ( d ) ) ) )" };

      for (int i = 0; i < stringsInL.length; i++)
      {
         Tree parseTree = parse(stringsInL[i]);

         try  // Create dot and png files from the AST.
         {
            // Create the (empty) dot file.
            String baseName = String.format("ParserL5_%02d_AST", i);
            java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot") );
            // Put dot commands into the dot file
            out.println("//  " + parseTree + "\n");
            out.println( Tree2dot.tree2dot(parseTree) + "\n" );
            out.close();
            // Create a command line for running dot.exe.
            String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }//main()

}//Parser_25
