/**
   This program produces a full parse tree instead of
   an abstract syntax tree as in example Parse_25.java.

   L = { a, a+a, a*a, a+(a+a), (a+a)*a, a*a+a, (a*a)*a, ... }

   '*' has higher precedence than '+',
   '+' is left associative,
   '*' is right associative.

    BNF:   expr -> expr '+' term | term       // expr is left recursive
           term -> factor '*' term | factor   // term is right recursive
         factor -> 'a' | '(' expr ')'


    EBNF:  expr -> term ( '+' term )*        // expr is iterative
           term -> factor [ '*' term ]       // term is right recursive
         factor -> 'a' | '(' expr ')'

   NOTE: We use the EBNF grammar to write the parser that builds
         the parse tree, but the parser should build the parse tree
         for the BNF grammar, NOT the EBNF grammar. The EBNF grammar
         does have its own parse tree, but its parse tree, by itself,
         does not specify left or right associativity. This is because
         the EBNF grammar can be derived from either a left or right
         recursive BNF grammar, so the EBNF grammar, by itself, does
         not specify associativity.
*/

public class Parser_25_ParseTree
{
   public static Tree parse(String stringInL)
   {
      Tokenizer tokens = new Tokenizer(stringInL);

      return getExpr(tokens);
   }//parse()


   public static Tree getExpr(Tokenizer tokens)
   {
      Tree result = new Tree( "Expr" );
      result.addSubTree(getTerm(tokens));         // consume a sub expression
      while( tokens.hasToken() && tokens.peekToken().equals("+") )
      {
         result = new Tree("Expr",
                           result,                    // old tree on the left
                           new Tree(tokens.nextToken()),     // consume a '+'
                           getTerm(tokens));      // consume a sub expression
      }
      return result;
   }//getExpr()


   public static Tree getTerm(Tokenizer tokens)
   {
      Tree result = new Tree( "Term" );;
      result.addSubTree( getFactor(tokens) );
      if( tokens.hasToken() && tokens.peekToken().equals("*") )
      {
         result.addSubTree( new Tree(tokens.nextToken()) ); // consume the '*'
         result.addSubTree( getTerm(tokens) );              // consume a sub expression
      }
      return result;
   }//getTerm()


   public static Tree getFactor(Tokenizer tokens)
   {
      Tree result = new Tree( "Factor" );
      String tk = tokens.peekToken();
      if( tk.equals("(") )             // conditionally consume more tokens
      {
         result.addSubTree( new Tree(tokens.nextToken()) ); // consume a '('
         result.addSubTree( getExpr(tokens) );              // consume a sub expression
         result.addSubTree( new Tree(tokens.nextToken()) ); // consume a ')'
      }
      else
      {
         result.addSubTree( new Tree(tokens.nextToken()) ); // consume the 'a'
      }
      return result;
   }//getFactor()


   public static void main(String[] args)
   {
      String[] stringsInL = { "a + b + c",
                              "a + ( b + c )",
                              "a * b + c",
                              "a * ( b + c )",
                              "a * b * c",
                              "( a * b ) * c",
                              "a + b + ( c + e + f ) + g + h",     // sum of 5 terms
                              "a + ( b + c ) * e * ( f + g + h )", // sum of 2 terms
                              "( a + b + c + e ) * f * g * h",     // product of 4 factors
                              "a * b * ( c * e + f + g + h ) ",    // product of 3 factors
                              "a * b * c + e + f * g * h",         // sum of 3 terms
                              "a * b * c * e * f * g * h",         // product of 7 factors
                              "a + b + c * d",     // put in lots of redundant parentheses
                              "( a + b ) + c * d",
                              "( a + b ) + ( c * d )",
                              "( a + b ) + ( ( c * d ) )",
                              "( ( a + b ) + ( ( c * d ) ) )",
                              "( ( ( a ) + ( b ) ) + ( ( ( c ) * ( d ) ) ) )" };

      for (int i = 0; i < stringsInL.length; i++)
      {
         Tree parseTree = parse(stringsInL[i]);

         try  // Create dot and png files from the parse tree.
         {
            // Create the (empty) dot file.
            String baseName = String.format("ParserL5_%02d_ParseTree", i);
            java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot") );
            // Put dot commands into the dot file
            out.println("//  " + parseTree + "\n");
            out.println( Tree2dot.tree2dot(parseTree) + "\n" );
            out.close();
            // Create a command line for running dot.exe.
            String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }//main()

}//Parser_25_ParseTree
