/**
   Here is an unambiguous, "left associative", BNF grammar
   (and also two EBNF grammars) for the language

      L = { a, a+a, a+a+a, a+a+a+a, ... }.


   BNF:    expr -> expr '+' 'a' | 'a'   // expr is left recursive


   EBNF:   expr -> [ expr '+' ] 'a'     // expr is still left recursive


   EBNF:   expr -> 'a' ( '+' 'a' )*     // we "eliminated" the recursion


   Using the BNF grammar we parse the string "a+a+a+a" as if it were
   "grouped" as ((a+a)+a)+a.

   We cannot write a recusive descent parser using the BNF grammar
   or the first EBNF grammar because the left recursion would cause
   infinite recursion (the getExpr() method would immediately, and
   repeatedly, call itself). But we can use the second EBNF grammar
   to write a parser.

   Recall that the second EBNF grammar was also derived from the right
   recursive BNF grammar for the right associative version of L.
   Suprisingly, we use this EBNF production to write a recursive descent
   parser that is left associative. But notice that the parser below is
   not recursive (it uses a while-loop instead of recursion) because
   the production is itself not recursive.

   It is important to realize that this EBNF grammar can be the grammar
   for either a right or left associative parser.
*/

public class Parser_3
{
   public static Tree parse(String stringInL)
   {
      Tokenizer tokens = new Tokenizer(stringInL);

      return getExpr(tokens);
   }//parse()


   public static Tree getExpr(Tokenizer tokens)
   {
      String token = tokens.nextToken();  // consume a 'a'
      Tree result = new Tree( token );
      while( tokens.hasToken() && tokens.peekToken().equals("+") )
      {
         result = new Tree(tokens.nextToken(),        // consume the '+'
                           result,                    // old tree on the left
                           new Tree(tokens.nextToken())); // consume a 'a'
      }
      return result;
   }//getExpr()



   public static void main(String[] args)
   {
      String[] stringsInL = { "a",
                              "a + a",
                              "a + a + a",
                              "a + a + a + a",
                              "a + b + c + d" };

      for (int i = 0; i < stringsInL.length; i++)
      {
         Tree parseTree = parse(stringsInL[i]);

         try  // Create dot and png files from the AST.
         {
            // Create the (empty) dot file.
            String baseName = String.format("Parser_3_%02d_AST", i);
            java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot") );
            // Put dot commands into the dot file
            out.println("//  " + parseTree + "\n");
            out.println( Tree2dot.tree2dot(parseTree) + "\n" );
            out.close();
            // Create a command line for running dot.exe.
            String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }//main()

}//Parser_3


/*
   NOTE1: This parser builds the AST from the bottom up.
   Notice how in line 52, the constructor call
         result = new Tree(token1, result, token2);
   builds the new (larger) AST by making the previous
   AST the left sub-tree.


   NOTE2: If we change the constructor call mentioned above so
   that it uses the previous AST as the right sub-tree of the
   new AST, like this
         result = new BTree(token1, token2, result);
   AND if we make the tokenizer read the input string from
   right to left, then this algorithm does a right associative
   parse of the expression. This shows that the Kleene star
   grammar that this algorithm was derived from
           expr -> a ( + a )*
   does not by itself define any associativity for the operator.
   The associativity doesn't come from the grammar in this case.
   it comes from how we choose to implement the grammar.
   (It's easy to make the "tokenizer" class, Tokenizer.java,
   read a string from right to left. Try it.)
*/
