/**
   This program produces a full parse tree instead of
   an abstract syntax tree as in example Parse_3.java.

      L = { a, a+a, a+a+a, a+a+a+a, ... }.


   BNF:    expr -> expr '+' 'a' | 'a'   // expr is left recursive

   EBNF:   expr -> 'a' ( '+' 'a' )*     // should parse as left associative

   NOTE: We use the EBNF grammar to write the parser that builds
         the parse tree, but the parser should build the parse tree
         for the BNF grammar, NOT the EBNF grammar. The EBNF grammar
         does have its own parse tree (see Parser_3_ParseTree_Iterative.java),
         but its parse tree, by itself, does not specify left or right
         associativity. This is because the EBNF grammar can be derived
         from either a left or right recursive BNF grammar, so the EBNF
         grammar, by itself, does not specify associativity.
*/

public class Parser_3_ParseTree
{
   public static Tree parse(String stringInL)
   {
      Tokenizer tokens = new Tokenizer(stringInL);

      return getExpr(tokens);
   }//parse()


   public static Tree getExpr(Tokenizer tokens)
   {
      Tree result = new Tree( "Expr" );
      String token = tokens.nextToken();  // consume a 'a'
      result.addSubTree(new Tree( token ));
      while( tokens.hasToken() && tokens.peekToken().equals("+") )
      {
         result = new Tree("Expr",
                           result,                 // old tree on the left
                           new Tree(tokens.nextToken()),  // consume a '+'
                           new Tree(tokens.nextToken())); // consume a 'a'
      }
      return result;
   }//getExpr()



   public static void main(String[] args)
   {
      String[] stringsInL = { "a",
                              "a + a",
                              "a + a + a",
                              "a + a + a + a",
                              "a + b + c + d" };

      for (int i = 0; i < stringsInL.length; i++)
      {
         Tree parseTree = parse(stringsInL[i]);

         try  // Create dot and png files from the parse tree.
         {
            // Create the (empty) dot file.
            String baseName = String.format("Parser_3_%02d_ParseTree", i);
            java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot") );
            // Put dot commands into the dot file
            out.println("//  " + parseTree + "\n");
            out.println( Tree2dot.tree2dot(parseTree) + "\n" );
            out.close();
            // Create a command line for running dot.exe.
            String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }//main()

}//Parser_3_ParseTree
