/**
   This example produces a parse tree based on the iterative
   EBNF grammar instead of the parse tree based on the BNF
   grammar as was produced in the program Parser_3_ParseTree.java.

      L = { a, a+a, a+a+a, a+a+a+a, ... }.

   BNF:    expr -> expr '+' 'a' | 'a'   // expr is left recursive

   EBNF:   expr -> 'a' ( '+' 'a' )*     // should parse as left associative

   In this program, the getExpr() method builds a parse tree that is
   literally based on this iterative EBNF production.
           expr -> 'a' ( '+' 'a' )*
   Notice that the right-hand-side of this production does not contain
   any non-terminal symbols. In a parse tree, a non-terminal sysbol is
   always the root of a new sub-tree for the parse tree. Since this
   production does not have any non-terminals on the rhs, this production
   will create a parse tree with no sub-trees (i.e., only only leaf nodes).
   In other words, the depth of the parse tree will always be one, no
   matter how many times the ('+' 'a')* gets iterated. So all of the
   parsed  '+' and 'a' tokens will be just below the Expr root of the
   parse tree. The more  "+ a" terms there are in the string being parsed,
   the wider (not deeper!) will be this parse tree. Since all of these
   parsed tokens are lined up below the root node, this parse tree does
   not really specify whether the '+' operations are being grouped as
   left or right associative.

   The difference between these parse trees and the parse trees based
   on the BNF grammar is most obvious when comparing parse trees for
   the strings "a+a", "a+a+a", and "a+a+a+a".
*/

public class Parser_3_ParseTree_Iterative
{
   public static Tree parse(String stringInL)
   {
      Tokenizer tokens = new Tokenizer(stringInL);

      return getExpr(tokens);
   }//parse()


   public static Tree getExpr(Tokenizer tokens)
   {
      Tree result = new Tree( "Expr" );
      String token = tokens.nextToken();  // consume a 'a'
      result.addSubTree(new Tree( token ));
      while( tokens.hasToken() && tokens.peekToken().equals("+") )
      {
         result.addSubTree(new Tree( tokens.nextToken() )); // consume a '+'
         result.addSubTree(new Tree( tokens.nextToken() )); // consume a 'a'
      }
      return result;
   }//getExpr()



   public static void main(String[] args)
   {
      String[] stringsInL = { "a",
                              "a + a",
                              "a + a + a",
                              "a + a + a + a",
                              "a + b + c + d" };

      for (int i = 0; i < stringsInL.length; i++)
      {
         Tree parseTree = parse(stringsInL[i]);

         try  // Create dot and png files from the parse tree.
         {
            // Create the (empty) dot file.
            String baseName = String.format("Parser_3_%02d_ParseTree_Iterative", i);
            java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot") );
            // Put dot commands into the dot file
            out.println("//  " + parseTree + "\n");
            out.println( Tree2dot.tree2dot(parseTree) + "\n" );
            out.close();
            // Create a command line for running dot.exe.
            String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }//main()

}//Parser_3_ParseTree_Iterative
