/*
   This example produces a parse tree based on the iterative
   EBNF grammar instead of the parse tree based on the BNF
   grammar as was produced in the program Parser_6_ParseTree.java.

    L = {a, (a), a+a, (a)+a, a+(a), (a+a), a+a+a, (a+a)+a, a+(a+a), ((a+a)+a)+a, ...}

    BNF:   expr -> expr '+' term | term  // expr is left recursive
           term -> 'a' | '(' expr ')'

    EBNF:  expr -> term ( '+' term )*    // expr is iterative
           term -> 'a' | '(' expr ')'

    In this program, the getExpr() method builds a parse tree that is
    literally based on this iterative EBNF production.
           expr -> term ( '+' term )*
    In the parse tree, the Expr node has as many Term subnodes as
    there are '+' tokens in the string the method is iterating
    over. These Term subnodes are all at the same level of the parse
    tree (just below Expr) so the parse tree does not really specify
    whether the '+' operations are being grouped as left or right
    associative.

    The difference between these parse trees and the parse trees based
    on the BNF grammar is most obvious when comparing parse trees for
    the strings "a+a", "a+a+a", and "a+a+a+a".
*/

public class Parser_6_ParseTree_Iterative
{
   public static Tree parse(String stringInL)
   {
      Tokenizer tokens = new Tokenizer(stringInL);

      return getExpr(tokens);
   }//parse()


   public static Tree getExpr(Tokenizer tokens)
   {
      Tree result = new Tree( "Expr" );;
      result.addSubTree(getTerm(tokens)); // consume a sub expression
      while( tokens.hasToken() && tokens.peekToken().equals("+") )
      {
         result.addSubTree(new Tree( tokens.nextToken() )); // consume the '+'
         result.addSubTree(getTerm(tokens));       // consume a sub expression
      }
      return result;
   }//getExpr()


   public static Tree getTerm(Tokenizer tokens)
   {
      Tree result = new Tree( "Term" );
      String tk = tokens.peekToken();
      if( tk.equals("(") )             // conditionally consume more tokens
      {
         result.addSubTree(new Tree( tokens.nextToken() )); // consume a '('
         result.addSubTree(getExpr(tokens));     // consume a sub expression
         result.addSubTree(new Tree( tokens.nextToken() )); // consume a ')'
      }
      else
      {
         result.addSubTree(new Tree( tokens.nextToken() )); // consume the 'a'
      }
      return result;
   }//getTerm()



   public static void main(String[] args)
   {
      String[] stringsInL = { "a",
                              "a + a",
                              "a + a + a",
                              "a + a + a + a",
                              "( a )",
                              "( a ) + a",
                              "a + ( a )",
                              "( a + a )",
                              "a + a + a",
                              "( a + b ) + c",
                              "a + ( b + c )",
                              "( ( a + b ) + c ) + d",
                              "a + ( b + ( c + d ) )",
                              "( ( a ) + ( ( b ) + ( ( c + d ) ) ) )" };

      for (int i = 0; i < stringsInL.length; i++)
      {
         Tree parseTree = parse(stringsInL[i]);

         try  // Create dot and png files from the "parse tree".
         {
            // Create the (empty) dot file.
            String baseName = String.format("Parser_6_%02d_ParseTree_Iterative", i);
            java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot") );
            // Put dot commands into the dot file
            out.println("//  " + parseTree + "\n");
            out.println( Tree2dot.tree2dot(parseTree) + "\n" );
            out.close();
            // Create a command line for running dot.exe.
            String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }//main()

}//Parser_6_ParseTree_Iterative
