/**
   NOTE: This parser creates an abstract syntax tree (or, an "expression
   tree") for each postfix expression. An abstract syntax tree is much
   less cluttered than a full "parse tree". Whereas a parse tree gives
   you the "syntactic" structure of an expression, an abstract syntax
   tree gives you the "semantic" structure of the expression.

   The static method parse() is a "shift-reduce" parser for strings
   that hold the postfix notation of an arithmetic expression.

   Here is the BNF grammar for postfix notation expressions.

     Postfix -> Postfix Postfix BinOp
              | Postfix UnOp
              | Number

       BinOp -> '+' | '-' | '*' | '/' | '^'

       UnOp  -> 'neg' | 'sqrt'

      Number -> any valid Java double

   The shift-reduce parser maintains a stack of partial (bottom-up)
   parse trees. When the parser sees a token that is a number, the
   parser pushes onto the stack a tree whose root is that number.
   (This is a "shift" step of the parser.)

   When the parser sees a token that is a binary operator, the parser
   pops two operand trees off the stack, forms them into a binary
   operator tree, and then pushes that new operator tree onto the stack.
   (This is a "reduce" step of the parser.)

   When the parser sees a unary operator token, the parser pops one
   operand tree off the stack, forms a unary operator tree, and then
   pushes the new unary operator tree onto the stack. (This is another
   "reduce" step of the parser.)

   Input strings are "tokenized" by the very bare bones tokenizer
   contained in the file Tokenizer.java. This tokenizer requires
   that all tokens in the input string be surrounded by white space
   (which makes the tokenizer almost trivial to write).
*/
import java.util.Stack;

public class Parse2AST
{
   private static Stack<Tree> stack = null;


   public static Tree parse(String postfix) throws ParseException
   {
      Tokenizer tokens = new Tokenizer(postfix); // defined in Tokenizer.java

      stack = new Stack<>();

      shiftReduce(tokens);

      if ( 1 != stack.size() ) // there should be only one tree in the stack
         throw new ParseException("unexpected items in stack: " + stack
                                                         + "\n" + tokens);
      return stack.pop();
   }//parse()



   private static void shiftReduce(Tokenizer tokens) throws ParseException
   {
      String token = null;

      while ( tokens.hasToken() )
      {
         token = tokens.nextToken();

         Tree newTree = new Tree(token);

         if ( "+-*/^".indexOf(token) != -1 ) // binary operator case
         {
            Tree op2 = stack.pop();   // throws EmptyStackException
            Tree op1 = stack.pop();   // throws EmptyStackException
            newTree.addSubTree( op1 );
            newTree.addSubTree( op2 );
         }
         else if ( "neg sqrt".indexOf(token) != -1 ) // unary operator case
         {
            Tree op1 = stack.pop();   // throws EmptyStackException
            newTree.addSubTree( op1 );
         }
         else  // number case
         {
            Double.parseDouble(token); // throws NumberFormatException
         }

         stack.push( newTree );
      }
    }//shiftReduce()

}//Parse2AST
